<?php

namespace BexioConnectorByHelvy;

/**
 * Admin API Synchronization
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class AdminAPISynchronization
{

    public function __construct()
    {
        add_action('rest_api_init', function () {
            // Usage quota
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/usage-quota', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_usage_quota'),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Orders sync state
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/orders/(?P<page>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_orders'),
                'args' => array(
                    'page' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Products sync state
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/products/(?P<page>\d+)', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_products'),
                'args' => array(
                    'page' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Sync product
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/product/(?P<product_id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_product'),
                'args' => array(
                    'product_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Sync order
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/payment', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order_payment'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/fulfillment', array(
                'methods' => 'POST',
                'callback' => array($this, 'sync_order_fulfillment'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Delete metadata
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/sync/order/(?P<order_id>\d+)/metadata', array(
                'methods' => 'DELETE',
                'callback' => array($this, 'delete_order_metadata'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            // Fetch PDFs
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/orders/(?P<order_id>\d+)/pdf', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_order_pdf'),
                'args' => array(
                    'order_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
            register_rest_route(BEXIO_CONNECTOR_BY_HELVY_REST_API_ROUTE, '/invoices/(?P<invoice_id>\d+)/pdf', array(
                'methods' => 'GET',
                'callback' => array($this, 'get_invoice_pdf'),
                'args' => array(
                    'invoice_id' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
                'permission_callback' => array($this, 'get_permission')
            ));
        });
    }

    /*
    * Fetching orders.
    */
    public function get_orders($request)
    {
        $params = $request->get_params();

        if (isset($params['page']) && is_numeric($params['page'])) {
            $page = intval($params['page']);
            $per_page = 10;

            $args = array(
                'limit'        => $per_page,
                'page'         => $page,
                'orderby'      => 'date',
                'order'        => 'DESC',
                'paginate'     => true,
            );

            // Fetch orders
            $orders = [];
            $total_orders = 0;
            if (!empty($params['search'])) {
                $search = ltrim($params['search'], '#');
                $search = intval($search);
                $order = wc_get_order($search);
                if ($order) {
                    $orders = [$order];
                    $total_orders = 1;
                } else {
                    $orders = [];
                }
            } else {
                $results = wc_get_orders($args);
                $orders = $results->orders;
                $total_orders = $results->total;
            }

            $order_data = array();

            foreach ($orders as $order) {
                if ($order && $order instanceof \WC_Order) {
                    $formatted_date = gmdate('M d, Y', strtotime($order->get_date_created()));
                    $customer_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();

                    $order_data[] = array(
                        'id'       => $order->get_id(),
                        'date'     => $formatted_date,
                        'customer' => $customer_name,
                        'total'    => $order->get_total(),
                        'currency'    => $order->get_currency(),
                        'metaBexioInvoiceID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id'),
                        'metaBexioInvoiceNumber'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_number'),
                        'metaBexioOrderID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id'),
                        'metaBexioOrderNumber'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_number'),
                        'metaBexioPaymentID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_payment_id'),
                        'metaBexioFulfillmentID'    => $order->get_meta(AdminAPISettings::$prefix . 'bexio_fulfillment_id'),
                        // 'orderStatus'   => $order->get_status(),
                    );
                }
            }

            $response = array(
                'total' => $total_orders,
                'data'  => $order_data,
            );

            return new \WP_REST_Response($response, 200);
        }

        return new \WP_REST_Response(array('message' => 'Something went wrong!'), 403);
    }

    /*
    * Fetching products.
    */
    public function get_products($request)
    {
        $params = $request->get_params();

        if (isset($params['page']) && is_numeric($params['page'])) {
            $page = intval($params['page']);
            $per_page = 10;

            $args = array(
                'limit'        => $per_page,
                'page'         => $page,
                'orderby'      => 'name',
                'order'        => 'ASC',
                'paginate'     => true,
            );

            if (!empty($params['search'])) {
                $args['sku'] = esc_attr($params['search']);
            }

            // Fetch products
            $results = wc_get_products(array_merge($args, array('status' => 'publish')));
            $products = $results->products;
            $total_products = $results->total;

            $product_data = array();

            foreach ($products as $product) {
                if ($product && $product instanceof \WC_Product) {
                    $formatted_date = gmdate('M d, Y', strtotime($product->get_date_created()));

                    $product_info = array(
                        'id'        => $product->get_id(),
                        'parent_id' => $product->get_id(),
                        'date'      => $formatted_date,
                        'name'      => $product->get_name(),
                        'sku'       => $product->get_sku(),
                        'price'     => $product->get_price(),
                        'currency'  => get_woocommerce_currency(),
                        'metaBexioProductID' => $product->get_meta(AdminAPISettings::$prefix . 'bexio_product_id'),
                        'variants' => []
                    );

                    // If variable product, list variables within the product info
                    if ($product->is_type('variable')) {
                        $variations = $product->get_children();
                        foreach ($variations as $variation_id) {
                            $variation = wc_get_product($variation_id);
                            if ($variation && $variation instanceof \WC_Product_Variation) {
                                $product_info['variants'][] = array(
                                    'id'        => $variation->get_id(),
                                    'parent_id' => $product->get_id(),
                                    'date'      => $formatted_date,
                                    'name'      => $variation->get_title() . ' - ' . wc_get_formatted_variation($variation, true, false),
                                    'sku'       => $variation->get_sku(),
                                    'price'     => $variation->get_price(),
                                    'currency'  => get_woocommerce_currency(),
                                    'metaBexioProductID' => $variation->get_meta(AdminAPISettings::$prefix . 'bexio_product_id'),
                                );
                            }
                        }
                    }

                    $product_data[] = $product_info;
                }
            }

            $response = array(
                'total' => $total_products,
                'data'  => $product_data,
            );

            return new \WP_REST_Response($response, 200);
        }

        return new \WP_REST_Response(array('message' => 'Something went wrong!'), 403);
    }

    /*
    * Retrieve the current usage quota
    */
    public function get_usage_quota($request)
    {
        $res = API::instance()->get_usage_quota();
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the given product (either simple product or variation) to bexio.
    */
    public function sync_product($request)
    {
        $params = $request->get_params();
        if (!isset($params['product_id']) && !is_numeric($params['product_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $product_id = intval($params['product_id']);

        $product = wc_get_product($product_id);
        if (!$product) {
            return new \WP_REST_Response(array('message' => 'Product not found'), 404);
        }

        $res = WCToBexio::instance()->sync_product($product, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the given order to bexio.
    */
    public function sync_order($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the payment of the given order to bexio.
    */
    public function sync_order_payment($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order_payment($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Sync the fulfillment of the given order to bexio.
    */
    public function sync_order_fulfillment($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);
        if (!$order) {
            new \WP_REST_Response(array('message' => 'Order not found'), 404);
        }

        $res = WCToBexio::instance()->sync_order_fulfillment($order, true);
        return new \WP_REST_Response($res);
    }

    /*
    * Delete metadata of the given order.
    */
    public function delete_order_metadata($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $order = wc_get_order($order_id);

        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_order_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_order_number');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_invoice_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_invoice_number');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_delivery_id');
        $order->delete_meta_data(AdminAPISettings::$prefix . 'bexio_delivery_number');
        $order->save_meta_data();

        return new \WP_REST_Response(["success" => true]);
    }

    /*
    * Retrieve the PDF of the given order.
    */
    public function get_order_pdf($request)
    {
        $params = $request->get_params();
        if (!isset($params['order_id']) && !is_numeric($params['order_id'])) {
            return new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $order_id = intval($params['order_id']);
        $pdf = API::instance()->get_order_pdf($order_id);
        if ($pdf instanceof \WP_Error) {
            return $pdf;
        }

        return new \WP_REST_Response($pdf);
    }

    /**
     * Retrieve the PDF of the given invoice
     */
    public function get_invoice_pdf($request) {
        $params = $request->get_params();
        if (!isset($params['invoice_id']) && !is_numeric($params['invoice_id'])) {
            return new \WP_REST_Response(array('message' => 'Missing order ID'), 403);
        }
        
        $invoice_id = intval($params['invoice_id']);
        $pdf = API::instance()->get_invoice_pdf($invoice_id);
        if ($pdf instanceof \WP_Error) {
            return $pdf;
        }

        return new \WP_REST_Response($pdf);
    }

    /**
     * Permission Callback
     **/
    public function get_permission()
    {
        if (current_user_can('administrator') || current_user_can('manage_woocommerce')) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Cloning is forbidden.
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), BEXIO_CONNECTOR_BY_HELVY_VERSION);
    }
}
